----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    21:32:02 07/09/2007 
-- Design Name: 
-- Module Name:    usb_sm - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity usb_sm is
    Port ( clk : in  STD_LOGIC;
           reset : in  STD_LOGIC;
           RXF_n : in  STD_LOGIC;
           TXE_n : in  STD_LOGIC;
           din : in  STD_LOGIC_VECTOR (7 downto 0);
           RD_n : out  STD_LOGIC;
           WR_n : out  STD_LOGIC;
           dir : out  STD_LOGIC;
           dout : out  STD_LOGIC_VECTOR (7 downto 0));
end usb_sm;

architecture Behavioral of usb_sm is

   type state_type is (idle,ready,rbyte,check,present,wbyte,hold); 
   signal state, next_state : state_type; 
   signal WR_n_i,RD_n_i,dir_i,capture_i,capture : std_logic;  
   signal value : std_logic_vector(7 downto 0);  

begin

   SYNC_PROC: process (clk)
   begin
      if (clk'event and clk = '1') then
         if (reset = '1') then -- asynch reset
            state <= idle;
            WR_n <= '1'; -- active low
            RD_n <= '1'; -- active low
            capture <= '0'; -- active hi
				dir <= '1'; -- 1 = read, 0 = write
        else -- clock rising edge found w/o reset
            state <= next_state;
            WR_n <= WR_n_i; -- FT2232D write strobe
            RD_n <= RD_n_i; -- FT2232D read strobe
            capture <= capture_i; -- store byte received
				dir <= dir_i; -- data bus direction control
         end if;        
      end if;
   end process;
 
   --MOORE State-Machine - Outputs based on state only
   OUTPUT_DECODE: process (state)
   begin
      if (state = ready or state = rbyte) then
         RD_n_i <= '0';
      else
         RD_n_i <= '1';
      end if;
      if (state = wbyte or state = hold) then
         WR_n_i <= '0';
      else
         WR_n_i <= '1';
      end if;
      if (state = present or state = wbyte or state = hold) then
         dir_i <= '0'; -- write
      else
         dir_i <= '1'; -- read
      end if;
      if (state = rbyte) then
         capture_i <= '1'; -- store value
      else
         capture_i <= '0';
      end if;
   end process;
 
   NEXT_STATE_DECODE: process (state, RXF_n, TXE_n)
   begin
      --declare default state for next_state to avoid latches
      next_state <= state;  --default is to stay in current state
      case (state) is
         when idle =>
            if RXF_n = '0' then
               next_state <= ready;
            end if;
         when ready => -- Read FIFO has a byte ready
            next_state <= rbyte;
         when rbyte => -- Store the read byte
            next_state <= check;
         when check => -- Check to see if WRITE FIFO ready
            if TXE_n = '0' then
               next_state <= present;
            end if;
         when present => -- present data on but
            next_state <= wbyte;
         when wbyte => -- assert write strobe
            next_state <= hold;
         when hold => -- hold cycle one clock
            next_state <= idle;
         when others =>
            next_state <= idle;
      end case;      
   end process;

   process (clk, reset)
   begin
     if reset='1' then  -- active high reset 
        value <= "00000000";
     elsif (clk'event and clk='1') then 
        if capture = '1' then 
           value <= din;
        end if; 
     end if;
   end process;
	
	dout <= value;
 
end Behavioral;

